"""
Script for calibration and imaging of EVN n14c3 data for J1849+3024/1848+283
NME_all.py or the instructions from the web page should first have been.
run to provide a file with Tsys/gain curve, initial delay and bandpass
calibration applied, 1848+283_J1849+3024.ms (or if this is not available,
use 1848+283_J1849+3024.ms.tgz)
The steps are outline on a web page.

See also NME_3C345_skeleton.py (or, if really stuck, NME_3C345.py) for
the other pair of sources.

Original by Anita Richards - amsr@jb.man.ac.uk April 2016
And updated by Jack Radcliffe - May 2019
"""

# Need: J1640+3946_3C345.ms flag_Tar1Ph1.flagcmd
# Data contains:

#  0   J1640+3946    16:40:29.632770 +39.46.46.02836 J2000  ph
#  1   3C345         16:42:58.809965 +39.48.36.99402 J2000  tar

# This script uses variables so that you could modify it to run on
# other similar data.
# NB in some plotms some antenna names are haard-wired for identifying
# bad data, which would have to be changed for other data sets

target='3C345'
phscal='J1640+3946'
inbase='n14c3'
antref='EF'                  # refant as origin of phase
flag_table='flag_TarPh.flagcmd'
parallel=False ## Only works for linux atm (need to init with mpicasa -n <cores> casa)

# Enter at the CASA prompt the number of the step(s) to run
# and then use execfile
# e.g.
# CASA <2>:mysteps=[1,2]
# CASA <3>:execfile('NME_3C345.py')

thesteps = []
step_title = {1: 'Inspect data',
			  2: 'Remove remaining bad data',
			  3: 'Make first image of phase calibrator',
			  4: 'Derive phase solution interval',
			  5: 'Derive delay and phase only corrections',
			  6: 'Apply all calibration and re-image phase cal',
			  7: 'Derive amplitude solutions',
			  8: 'Apply solutions and re-image phase cal',
			  9: 'Apply calibration to target and split',
			  10: 'First image of target source',
			  11: 'Phase self-calibrate target and apply',
			  12: 'Image phase-self-calibrated target',
			  13: 'Amp self calibrate target and apply',
			  14: 'Image amp and phase self-calibrated target',
			  15: 'Split calibrated data',
			  16: 'Re-weigh data',
			  17: 'Image with natural and uniform weighting'
}

thesteps = []
try:
	print('List of steps to be executed ...', mysteps)
	thesteps = mysteps
except:
	print('global variable mysteps not set')

print(mysteps)

if (thesteps==[]):
	thesteps = range(0,len(step_title))
	print('Executing all steps: ', thesteps)


### 1) Inspect data to remove remaining bad data
mystep = 1

if(mystep in thesteps):
	print('Step ', mystep, step_title[mystep])

	plotms(vis='%s_%s.ms'%(phscal,target),
		   xaxis='time',
		   yaxis='amp',
		   field='%s,%s'%(phscal,target),
		   correlation='RR,LL',
		   coloraxis='field',
		   avgchannel='32',
		   antenna='*&*',
		   iteraxis='baseline')

### 2) Apply flagtable and check bad data is gone
mystep = 2

if(mystep in thesteps):
	print('Step ', mystep, step_title[mystep])

	flagmanager(vis='%s_%s.ms'%(phscal,target),
				mode='save',versionname='pre_flagging')

	flagdata(vis='%s_%s.ms'%(phscal,target),
			 mode='list',
			 inpfile=flag_table)

	# Check the bad data have gone
	plotms(vis='%s_%s.ms'%(phscal,target),
		   xaxis='time',
		   yaxis='amp',
		   ydatacolumn='data',
		   field=phscal,
		   antenna='%s&*'%antref,
		   correlation='RR,LL',
		   coloraxis='antenna2',
		   avgchannel='32',
		   plotfile='%s_%s.pre-amp-cal.png'%(phscal,target),
		   overwrite=True)

### 3) Initial imaging from fringe fitting of phase calibrator
mystep = 3

if(mystep in thesteps):
	print('Step ', mystep, step_title[mystep])

	os.system('rm -r %s_phs_ref.*'%phscal)
	tclean(vis='%s_%s.ms'%(phscal,target),
		   cell='0.24mas',
		   field=phscal,
		   imagename='%s_phs_ref'%phscal,
		   imsize=[640,640],
		   deconvolver='clark',
		   niter=1000,
		   cycleniter=50,
		   interactive=True,
		   parallel=parallel)

	ft(vis='%s_%s.ms'%(phscal,target),
	   field=phscal,
	   model='%s_phs_ref.model'%phscal,
	   usescratch=True)

	# A source! with mainly symmetric artefacts characteristic of amplitude errors.
	# Measure the signal-to-noise
	rms=imstat(imagename='%s_phs_ref.image'%phscal,
			   box='60,60,580,240')['rms'][0]
	peak=imstat(imagename='%s_phs_ref.image'%phscal,
			   box='300,300,340,340')['max'][0]

	print('Peak %6.3f, rms %6.3f, S/N %6.0f' % (peak, rms, peak/rms))

### 4) Get the solution interval for phase calibrator
mystep = 4

if(mystep in thesteps):
	print('Step ', mystep, step_title[mystep])

	# Plot target and phase-ref, phase against time, averaging channels
	plotms(vis='%s_%s.ms'%(phscal,target),
		   xaxis='time',
		   yaxis='phase',
		   spw='1',
		   field='%s,%s'%(phscal,target),
		   correlation='RR,LL',
		   coloraxis='field',
		   avgchannel='32',
		   antenna='%s&*'%antref,
		   iteraxis='baseline')

### 5) Derive solutions and plot
mystep = 5

if(mystep in thesteps):
	print('Step ', mystep, step_title[mystep])

	os.system('rm -rf %s_%s.ms.K'%(phscal,target))
	gaincal(vis='%s_%s.ms'%(phscal,target),
			caltable='%s_%s.K'%(phscal,target),
			field=phscal,
			solint='60s',
			refant=antref,
			gaintype='K',
			parang=False)

	#plotcal(caltable='%s_%s.K'%(phscal,target),
	#		xaxis='time',yaxis='delay',
	#		iteration='antenna',
	#		subplot=321,
	#		figfile='%s_%s_K.png'%(phscal,target))

	# Time-dependent phase correction of phase cal, applying delay solutions

	os.system('rm -rf %s_%s.p'%(phscal,target))
	gaincal(vis='%s_%s.ms'%(phscal,target),
			caltable='%s_%s.p'%(phscal,target),
			field=phscal,
			solint='20s',
			refant=antref,
			gaintype='G',
			calmode='p',
			gaintable=['%s_%s.K'%(phscal,target)],
			interp=['nearest'],
			parang=False)

	#plotcal(caltable='%s_%s.p'%(phscal,target),
	#		xaxis='time',yaxis='phase',
	#		iteration='antenna',
	#		subplot=321,
	#		figfile='%s_%s.p.png'%(phscal,target))


### 6) Apply phase solutions and clean phase-reference
mystep = 6

if(mystep in thesteps):
	print('Step ', mystep, step_title[mystep])

	# Apply these solutions to the phase cal and image
	applycal(vis='%s_%s.ms'%(phscal,target),
			 field=phscal,
			 gaintable=['%s_%s.K'%(phscal,target),'%s_%s.p'%(phscal,target)],
			 gainfield=[phscal,phscal],
			 interp=['nearest','linear'],
			 applymode='calflag',
			 parang=False)

	os.system('rm -r %s_p.*'%phscal)
	tclean(vis='%s_%s.ms'%(phscal,target),
		   cell='0.24mas',
		   field=phscal,
		   imagename='%s_p'%phscal,
		   imsize=[640,640],
		   deconvolver='clark',
		   niter=1000,
		   cycleniter=50,
		   interactive=True,
		   parallel=parallel)

	ft(vis='%s_%s.ms'%(phscal,target),
	   field=phscal,
	   model='%s_p.model'%phscal,
	   usescratch=True)

	rms=imstat(imagename='%s_p.image'%phscal,
			   box='60,60,580,240')['rms'][0]
	peak=imstat(imagename='%s_p.image'%phscal,
			   box='300,300,340,340')['max'][0]

	print('Peak %6.3f, rms %6.3f, S/N %6.0f' % (peak, rms, peak/rms))

### 7) Derive amplitude solutions
mystep = 7

if(mystep in thesteps):
	print('Step ', mystep, step_title[mystep])

	os.system('rm -rf %s_%s.ap'%(phscal,target))
	gaincal(vis='%s_%s.ms'%(phscal,target),
			caltable='%s_%s.ap'%(phscal,target),
			field=phscal,
			solint='60s',
			refant=antref,
			gaintype='G',
			minblperant=4,
			solnorm=True,
			calmode='ap',
			gaintable=['%s_%s.K'%(phscal,target),'%s_%s.p'%(phscal,target)],
			interp=['nearest','nearest'],
			parang=False)

	plotcal(caltable='%s_%s.ap'%(phscal,target),
			xaxis='time',yaxis='amp',
			iteration='antenna',
			subplot=321,
			figfile='%s_%s.ap.png'%(phscal,target))

### 8) Apply amp solutions and clean phase reference again
mystep = 8

if(mystep in thesteps):
	print('Step ', mystep, step_title[mystep])

	# Apply these solutions to the phase cal and image
	applycal(vis='%s_%s.ms'%(phscal,target),
			 field=phscal,
			 gaintable=['%s_%s.K'%(phscal,target),'%s_%s.p'%(phscal,target),'%s_%s.ap'%(phscal,target)],
			 gainfield=[phscal,phscal,phscal],
			 interp=['nearest','nearest','nearest'],
			 applymode='calflag',
			 parang=False)

	os.system('rm -r %s_ap.*'%phscal)
	tclean(vis='%s_%s.ms'%(phscal,target),
		   cell='0.24mas',
		   field=phscal,
		   imagename='%s_ap'%phscal,
		   imsize=[640,640],
		   deconvolver='clark',
		   niter=1000,
		   cycleniter=50,
		   interactive=True,
		   parallel=parallel)

	rms=imstat(imagename='%s_ap.image'%phscal,
			   box='60,60,580,240')['rms'][0]
	peak=imstat(imagename='%s_ap.image'%phscal,
			   box='300,300,340,340')['max'][0]

	print('Peak %6.3f, rms %6.3f, S/N %6.0f' % (peak, rms, peak/rms))

### 9) Apply solutions to target and split
mystep = 9

if(mystep in thesteps):
	print('Step ', mystep, step_title[mystep])

	# Apply these solutions to the phase cal and image
	applycal(vis='%s_%s.ms'%(phscal,target),
			 field='',
			 gaintable=['%s_%s.K'%(phscal,target),'%s_%s.p'%(phscal,target),'%s_%s.ap'%(phscal,target)],
			 gainfield=[phscal,phscal,phscal],
			 interp=['linear','linear','linear'],
			 applymode='calflag',
			 parang=False)

	os.system('rm -r %s.ms'%target)
	split(vis='%s_%s.ms'%(phscal,target),
		  field=target,
		  outputvis='%s.ms'%target)

### 10) Image target (set timerange due to non-phase referenced parts
mystep = 10

if(mystep in thesteps):
	print('Step ', mystep, step_title[mystep])

	os.system('rm -r %s_phs_ref.*'%target)
	tclean(vis='%s.ms'%(target),
		   cell='0.24mas',
		   field=target,
		   imagename='%s_phs_ref'%target,
		   timerange='12:15:00~13:17:00',
		   imsize=[640,640],
		   deconvolver='clark',
		   niter=1000,
		   interactive=True,
		   parallel=parallel)

	ft(vis='%s.ms'%(target),
	   model='%s_phs_ref.model'%target,
	   usescratch=True)

	rms=imstat(imagename='%s_phs_ref.image'%target,
			   box='60,60,580,240')['rms'][0]
	peak=imstat(imagename='%s_phs_ref.image'%target,
			   box='300,300,340,340')['max'][0]
	print('Peak %6.3f, rms %6.3f, S/N %6.0f' % (peak, rms, peak/rms))

### 11) Get phase corrections
mystep = 11

if(mystep in thesteps):
	print('Step ', mystep, step_title[mystep])

	os.system('rm -rf %s.p'%(target))
	gaincal(vis='%s.ms'%(target),
			caltable='%s.p'%(target),
			solint='20s',
			refant=antref,
			gaintype='G',
			minsnr=2,
			calmode='p',
			parang=False)

	plotcal(caltable='%s.p'%(target),
			xaxis='time',yaxis='phase',
			iteration='antenna',
			subplot=321,
			figfile='%s.p.png'%(target))

### 12) Apply and image
mystep = 12

if(mystep in thesteps):
	print('Step ', mystep, step_title[mystep])

	# Apply these solutions to the phase cal and image
	applycal(vis='%s.ms'%(target),
			 field=target,
			 gaintable=['%s.p'%(target)],
			 gainfield=[target],
			 interp=['linear'],
			 applymode='calflag',
			 parang=False)

	os.system('rm -r %s_sc_p.*'%target)
	tclean(vis='%s.ms'%(target),
		   cell='0.24mas',
		   imagename='%s_sc_p'%target,
		   imsize=[640,640],
		   deconvolver='clark',
		   niter=1000,
		   interactive=True,
		   parallel=parallel)

	ft(vis='%s.ms'%(target),
	   field=target,
	   model='%s_sc_p.model'%target,
	   usescratch=True)

	rms=imstat(imagename='%s_sc_p.image'%target,
			   box='60,60,580,240')['rms'][0]
	peak=imstat(imagename='%s_sc_p.image'%target,
			   box='300,300,340,340')['max'][0]

	print('Peak %6.3f, rms %6.3f, S/N %6.0f' % (peak, rms, peak/rms))

### 13) Derive amplitude solutions
mystep = 13

if(mystep in thesteps):
	print('Step ', mystep, step_title[mystep])

	os.system('rm -rf %s.ap'%(target))
	gaincal(vis='%s.ms'%(target),
			caltable='%s.ap'%(target),
			field=target,
			solint='inf',
			refant=antref,
			gaintype='G',
			minblperant=4,
			minsnr=2,
			solnorm=True,
			calmode='ap',
			gaintable=['%s.p'%(target)],
			interp=['linear'],
			parang=False)

	plotcal(caltable='%s.ap'%(target),
			xaxis='time',yaxis='amp',
			iteration='antenna',
			subplot=321,
			figfile='%s.ap.png'%(target))

### 14) Apply and image
mystep = 14

if(mystep in thesteps):
	print('Step ', mystep, step_title[mystep])

	# Apply these solutions to the phase cal and image
	applycal(vis='%s.ms'%(target),
			 field=target,
			 gaintable=['%s.p'%(target),'%s.ap'%target],
			 gainfield=[target,target],
			 interp=['linear','linear'],
			 applymode='calflag',
			 parang=False)

	os.system('rm -r %s_sc_ap.*'%target)
	tclean(vis='%s.ms'%(target),
		   cell='0.24mas',
		   imagename='%s_sc_ap'%target,
		   imsize=[640,640],
		   deconvolver='clark',
		   niter=1000,
		   interactive=True,
		   parallel=parallel)

	rms=imstat(imagename='%s_sc_ap.image'%target,
			   box='60,60,580,240')['rms'][0]
	peak=imstat(imagename='%s_sc_ap.image'%target,
			   box='300,300,340,340')['max'][0]

	print('Peak %6.3f, rms %6.3f mJy/b, S/N %6.0f' % (peak, rms*1e3, peak/rms))

### 15) Split calibrated data
mystep = 15

if(mystep in thesteps):
	print('Step ', mystep, step_title[mystep])
	os.system('rm -r %s_calibrated.ms'%target)
	split(vis='%s.ms'%target,
		  outputvis='%s_calibrated.ms'%target)

### 16) Re-weigh data
mystep = 16

if(mystep in thesteps):
	print('Step ', mystep, step_title[mystep])
	statwt(vis='%s_calibrated.ms'%target,
		   datacolumn='data')
	plotms(vis='%s_calibrated.ms'%target,
		   xaxis='uvwave',
		   yaxis='wt',
		   antenna='ON&*',
		   correlation='LL,RR',
		   coloraxis='antenna2',
		   plotfile='')

### 17) Clean image using uniform, natural and robust weighting
mystep = 17
if(mystep in thesteps):
	print('Step ', mystep, step_title[mystep])

	os.system('rm -r %s_sc_uniform.*'%target)
	tclean(vis='%s_calibrated.ms'%(target),
		   cell='0.1mas',
		   imagename='%s_sc_uniform'%target,
		   weighting='uniform',
		   imsize=[640,640],
		   deconvolver='clark',
		   niter=1000,
		   interactive=True,
		   parallel=parallel)

	rms=imstat(imagename='%s_sc_uniform.image'%target,
			   box='60,60,580,240')['rms'][0]
	peak=imstat(imagename='%s_sc_uniform.image'%target,
			   box='300,300,340,340')['max'][0]

	print('Peak %6.3f, rms %6.3f, S/N %6.0f' % (peak, rms, peak/rms))


	os.system('rm -r %s_sc_natural.*'%target)
	tclean(vis='%s_calibrated.ms'%(target),
		   cell='0.24mas',
		   imagename='%s_sc_natural'%target,
		   weighting='natural',
		   imsize=[640,640],
		   deconvolver='clark',
		   niter=1000,
		   interactive=True,
		   parallel=parallel)

	rms=imstat(imagename='%s_sc_natural.image'%target,
			   box='60,60,580,240')['rms'][0]
	peak=imstat(imagename='%s_sc_natural.image'%target,
			   box='300,300,340,340')['max'][0]

	print('Peak %6.3f, rms %6.3f, S/N %6.0f' % (peak, rms, peak/rms))

	os.system('rm -r %s_sc_robust0.*'%target)
	tclean(vis='%s_calibrated.ms'%(target),
		   cell='0.1mas',
		   imagename='%s_sc_robust0.5'%target,
		   weighting='briggs',
		   robust=0.5,
		   imsize=[640,640],
		   deconvolver='clark',
		   niter=1000,
		   interactive=True,
		   parallel=parallel)

	rms=imstat(imagename='%s_sc_robust0.5.image'%target,
			   box='60,60,580,240')['rms'][0]
	peak=imstat(imagename='%s_sc_robust0.5.image'%target,
			   box='300,300,340,340')['max'][0]

	print('Peak %6.3f, rms %6.3f, S/N %6.0f' % (peak, rms, peak/rms))
